/**
 * Copyright (C) 2015 Wasabeef
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var RE = {};

RE.currentSelection = {
    "startContainer": 0,
    "startOffset": 0,
    "endContainer": 0,
    "endOffset": 0};

RE.editor = document.getElementById('editor');

document.addEventListener("selectionchange", function() { RE.backuprange(); });

// Initializations
RE.callback = function() {
    var ranges = [
             '\ud83c[\udf00-\udfff]',
             '\ud83d[\udc00-\ude4f]',
             '\ud83d[\ude80-\udeff]'
         ];

        var oldlen=RE.editor.innerHTML;
        var newlen=RE.editor.innerHTML.replace(new RegExp(ranges.join('|'), 'g'), '');

        if(oldlen!=newlen){
            RE.backuprange();
            RE.editor.innerHTML =  RE.editor.innerHTML.replace(new RegExp(ranges.join('|'), 'g'), '');
            RE.restorerange();
        }

        window.location.href = "re-callback://" + encodeURI(RE.getHtml());
}

RE.scrollToBottom = function() {
    document.body.scrollTop = $(document).height();
}

RE.scrollToCursor = function() {

//window.getSelection().baseNode.parentNode
    if ('#text' == window.getSelection().baseNode.nodeName) {
        console.log('#text');
        document.body.scrollTop = $(window.getSelection().baseNode.parentNode).offset().top;
    } else {
        console.log('invoke scrollToCursor..........top + height = ' + $(window.getSelection().baseNode).offset().top + $(window.getSelection().baseNode).offset().height);
        document.body.scrollTop = $(window.getSelection().baseNode).offset().top;
    }
}

RE.setHtml = function(contents) {
    RE.editor.innerHTML = decodeURIComponent(contents.replace(/\+/g, '%20'));

    $('#editor').delegate('.img-post .del' , 'click' , function(e){
    	$(e.target).parent().remove();
    	RE.deleteItem();
    })

    $('.audio-play-strip').each(function(item){
        console.log("item = " + item);
    	audioStrip($(this));
    })
}

RE.hideChannelArrow = function() {
    $('#channelArrow').hide();
}


RE.getHtml = function() {
    var ranges = [
            '\ud83c[\udf00-\udfff]',
            '\ud83d[\udc00-\ude4f]',
            '\ud83d[\ude80-\udeff]'
        ];

        return RE.editor.innerHTML.replace(new RegExp(ranges.join('|'), 'g'), '');
}

RE.getText = function() {
    return RE.editor.innerText;
}

RE.setBaseTextColor = function(color) {
    RE.editor.style.color  = color;
}

RE.setBaseFontSize = function(size) {
    RE.editor.style.fontSize = size;
}


RE.setPadding = function(left, top, right, bottom) {
  RE.editor.style.paddingLeft = left;
  RE.editor.style.paddingTop = top;
  RE.editor.style.paddingRight = right;
  RE.editor.style.paddingBottom = bottom;
}


RE.setBackgroundColor = function(color) {
    document.body.style.backgroundColor = color;
}

RE.setBackgroundImage = function(image) {
    RE.editor.style.backgroundImage = image;
}

RE.setWidth = function(size) {
    RE.editor.style.minWidth = size;
}


RE.setHeight = function(size) {
    RE.editor.style.height = size;
}


RE.setTextAlign = function(align) {
    RE.editor.style.textAlign = align;
}

RE.setVerticalAlign = function(align) {
    RE.editor.style.verticalAlign = align;
}

RE.setPlaceholder = function(placeholder) {
    RE.editor.setAttribute("placeholder", placeholder);
}


RE.undo = function() {
    document.execCommand('undo', false, null);
}

RE.redo = function() {
    document.execCommand('redo', false, null);
}

RE.setBold = function() {
    document.execCommand('bold', false, null);
}

RE.setItalic = function() {
    document.execCommand('italic', false, null);
}

RE.setSubscript = function() {
    document.execCommand('subscript', false, null);
}

RE.setSuperscript = function() {
    document.execCommand('superscript', false, null);
}

RE.setStrikeThrough = function() {
    document.execCommand('strikeThrough', false, null);
}

RE.setUnderline = function() {
    document.execCommand('underline', false, null);
}

RE.setBullets = function() {
    document.execCommand('InsertUnorderedList', false, null);
}

RE.setNumbers = function() {
    document.execCommand('InsertOrderedList', false, null);
}

RE.setTextColor = function(color) {
//    RE.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('foreColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}

RE.setTextBackgroundColor = function(color) {
//    RE.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('hiliteColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}

RE.setFontSize = function(fontSize){
    document.execCommand("fontSize", false, fontSize);
}

RE.setHeading = function(heading) {
    document.execCommand('formatBlock', false, '<h'+heading+'>');
}

RE.setIndent = function() {
    document.execCommand('indent', false, null);
    RE.enabledEditingItem();
}

RE.setOutdent = function() {
    document.execCommand('outdent', false, null);
    RE.enabledEditingItem();
}

RE.setJustifyLeft = function() {
    if (!RE.isEditorFocus()) { return; }
    document.execCommand('justifyLeft', false, null);
}

RE.setJustifyCenter = function() {
    if (!RE.isEditorFocus()) { return; }

    document.execCommand('justifyCenter', false, null);
}

RE.setJustifyRight = function() {
    if (!RE.isEditorFocus()) { return; }

    document.execCommand('justifyRight', false, null);
}

RE.setJustifyFull = function() {
    if (!RE.isEditorFocus()) { return; }

    document.execCommand('justifyFull', false, null);
}

RE.setBlockquote = function() {
    if (!RE.isEditorFocus()) { return; }

    document.execCommand('formatBlock', false, '<blockquote>');
}

RE.insertImage = function(url, alt) {
     var html = '<br/><img  src="' + url + '" alt="' + alt + '" ></img><br/>';
     RE.insertHTML(html);
}

RE.onInsertImage = function(id) {
    //console.log('invoke onInsertImage.......... id = ' + id)
    console.log('invoke onInsertImage.......... top = ' + $('#'+ id).offset().top )
    console.log('invoke onInsertImage.......... height = ' + $('#'+ id).offset().height )
    document.body.scrollTop = $('#'+ id).offset().top + $('#'+ id).offset().height
}

RE.getCharacterPrecedingCaret = function(containerEl) {
    var precedingChar = "", sel, range, precedingRange;
    if (window.getSelection) {
        sel = window.getSelection();
        if (sel.rangeCount > 0) {
            range = sel.getRangeAt(0).cloneRange();
            range.collapse(true);
            range.setStart(containerEl, 0);
            precedingChar = range.toString().slice(-10);
        }
    } else if ( (sel = document.selection) && sel.type != "Control") {
        range = sel.createRange();
        precedingRange = range.duplicate();
        precedingRange.moveToElementText(containerEl);
        precedingRange.setEndPoint("EndToStart", range);
        precedingChar = precedingRange.text.slice(-10);
    }
    return precedingChar;
}
//插入分割线
RE.insertHr = function() {
    var html = '<br/><hr color=#e2e2e2 size=1 /><br/>';
    RE.insertHTML(html);
}
 /**
     * 根据url生成视频预览
     * 修改：处理通用视频代码iframe   BY oceanking 20150309
     * @param url
     */
 RE.insertVideo =  function(url){
      var html = '<br/><iframe src="'+url+'" width="450" height="372" frameborder="0"></iframe><br/>';
      RE.insertHTML(html);
 }

RE.insertAudio = function(url, duration) {
    if (!RE.isEditorFocus()) { return; }

    var id = new Date().getTime();
    var html = '<div audio="start" id="'+ id +'" data-length="' + duration + '" data-clicks="1" data-ctime="0" isaudioready="no" ' +
                    'src="' + url + '"' +
                    'class="audio-play-strip icon-playing">' +
                    '<a data-status="0" class="icon"></a>' +
                    '<div class="progress">' +
                    '<div class="gray"></div>' +
                    '<div class="red"></div>' +
                    '<a class="circle"><i class="loading"></i></a></div><span class="time"></span><i class="del"></i><span audio="end"></span></div><br/>';

    //nodeType == 3 表示text
    if (RE.currentSelection.startContainer.nodeType == 3
        && RE.currentSelection.startOffset > 0) {
        html = '<br/>' + html;
    }

    RE.insertHTML(html);
    RE.audioStrip(id);
    RE.scrollToBottom();
}

RE.setChannelText = function(text) {
    $('#channel').text(text);
}

RE.audioStrip = function(id) {
    audioStrip($('#' + id))
}

RE.isEditorFocus = function() {
    return $('#editor').is(':focus');
}


RE.insertHTML = function(html) {
    RE.restorerange();
    document.execCommand('insertHTML', false, html);
}

RE.insertLink = function(url, title) {
    RE.restorerange();
    var sel = document.getSelection();
    if (sel.toString().length == 0) {
        document.execCommand("insertHTML",false,"<a href='"+url+"'>"+title+"</a>");
    } else if (sel.rangeCount) {
       var el = document.createElement("a");
       el.setAttribute("href", url);
       el.setAttribute("title", title);

       var range = sel.getRangeAt(0).cloneRange();
       range.surroundContents(el);
       sel.removeAllRanges();
       sel.addRange(range);
   }
   RE.callback();
}

RE.setTodo = function(text) {
    var html = '<input type="checkbox" name="'+ text +'" value="'+ text +'"/> &nbsp;';
    document.execCommand('insertHTML', false, html);
}

RE.prepareInsert = function() {
    RE.backuprange();
}

RE.backuprange = function(){
    var selection = window.getSelection();
    if (selection.rangeCount > 0) {
      var range = selection.getRangeAt(0);
      RE.currentSelection = {
          "startContainer": range.startContainer,
          "startOffset": range.startOffset,
          "endContainer": range.endContainer,
          "endOffset": range.endOffset};
    }
}

RE.restorerange = function(){
    var selection = window.getSelection();
    selection.removeAllRanges();
    var range = document.createRange();
    range.setStart(RE.currentSelection.startContainer, RE.currentSelection.startOffset);
    range.setEnd(RE.currentSelection.endContainer, RE.currentSelection.endOffset);
    selection.addRange(range);
}

RE.enabledEditingItem=function(){
var items = [];

    if (document.queryCommandState('bold')) {
        items.push('bold');
    }
    if (document.queryCommandState('italic')) {
        items.push('italic');
    }
    if (document.queryCommandState('subscript')) {
        items.push('subscript');
    }
    if (document.queryCommandState('superscript')) {
        items.push('superscript');
    }
    if (document.queryCommandState('strikeThrough')) {
        items.push('strikeThrough');
    }
    if (document.queryCommandState('underline')) {
        items.push('underline');
    }
    if (document.queryCommandState('insertOrderedList')) {
        items.push('orderedList');
    }
    if (document.queryCommandState('insertUnorderedList')) {
        items.push('unorderedList');
    }

    if (document.queryCommandState('justifyLeft')) {
        items.push('justifyLeft');
    }

    if (document.queryCommandState('justifyCenter')) {
        items.push('justifyCenter');
    }

    if (document.queryCommandState('justifyRight')) {
        items.push('justifyRight');
    }
    if (document.queryCommandState('justifyFull')) {
        items.push('justifyFull');
    }
    if (document.queryCommandState('insertHorizontalRule')) {
        items.push('horizontalRule');
    }
    if(document.queryCommandValue('fontSize')){
        items.push('fontSize&_&'+document.queryCommandValue('fontSize'));
    }
    if(document.queryCommandValue('backColor')){
        items.push('backcolor&_&'+document.queryCommandValue('backColor'));
    }
    if(document.queryCommandValue('foreColor')){
        items.push('foreColor&_&'+document.queryCommandValue('foreColor'));
    }
    var formatBlock = document.queryCommandValue('formatBlock');
    if (formatBlock.length > 0) {
        items.push(formatBlock);
    }
    var ranges = [
            '\ud83c[\udf00-\udfff]',
            '\ud83d[\udc00-\ude4f]',
            '\ud83d[\ude80-\udeff]'
    ];

    window.location.href = "re-state://" + encodeURI(items.join('#'))+"@_@"+encodeURI(RE.getHtml());
}
RE.enabledEditingItems = function(e) {
    var items = [];

    if (document.queryCommandState('bold')) {
        items.push('bold');
    }
    if (document.queryCommandState('italic')) {
        items.push('italic');
    }
    if (document.queryCommandState('subscript')) {
        items.push('subscript');
    }
    if (document.queryCommandState('superscript')) {
        items.push('superscript');
    }
    if (document.queryCommandState('strikeThrough')) {
        items.push('strikeThrough');
    }
    if (document.queryCommandState('underline')) {
        items.push('underline');
    }
    if (document.queryCommandState('insertOrderedList')) {
        items.push('orderedList');
    }
    if (document.queryCommandState('insertUnorderedList')) {
        items.push('unorderedList');
    }

    if (document.queryCommandState('justifyLeft')) {
        items.push('justifyLeft');
    }

    if (document.queryCommandState('justifyCenter')) {
        items.push('justifyCenter');
    }

    if (document.queryCommandState('justifyRight')) {
        items.push('justifyRight');
    }
    if (document.queryCommandState('justifyFull')) {
        items.push('justifyFull');
    }
    if (document.queryCommandState('insertHorizontalRule')) {
        items.push('horizontalRule');
    }
    if(document.queryCommandValue('fontSize')){
        items.push('fontSize&_&'+document.queryCommandValue('fontSize'));
    }
    if(document.queryCommandValue('backColor')){
        items.push('backcolor&_&'+document.queryCommandValue('backColor'));
    }
    if(document.queryCommandValue('foreColor')){
        items.push('foreColor&_&'+document.queryCommandValue('foreColor'));
    }
    var formatBlock = document.queryCommandValue('formatBlock');
    if (formatBlock.length > 0) {
        items.push(formatBlock);
    }

    if(e.which==13){
        items.push("enter");
    }
//    var editorValue = RE.editor.innerHTML;
//    var actualText_num = new RegExp('<.+?>').replace(editorValue,'');
//
//    alert("不能超过"+actualText_num+"个字!");

    var ranges = [
        '\ud83c[\udf00-\udfff]',
        '\ud83d[\udc00-\ude4f]',
        '\ud83d[\ude80-\udeff]'
    ];

    window.location.href = "re-state://" + encodeURI(items.join('#'))+"@_@"+encodeURI(RE.getHtml());
}


RE.focus = function() {
    var range = document.createRange();
    range.selectNodeContents(RE.editor);
    range.collapse(false);
    var selection = window.getSelection();
    selection.removeAllRanges();
    selection.addRange(range);
    RE.editor.focus();
    RE.enabledEditingItem();
}

RE.blurFocus = function() {
    RE.editor.blur();
}

RE.placeCaretAtEndEditor = function() {
    var el = RE.editor;
    RE.placeCaretAtEnd(el);
}

RE.getCursortPosition = function(ctrl) {//获取光标位置函数
   	var CaretPos = 0;	// IE Support
   	if (document.selection) {
   	    ctrl.focus ();
   		var Sel = document.selection.createRange ();
   		Sel.moveStart ('character', -ctrl.value.length);
   		CaretPos = Sel.text.length;
   	}
   	// Firefox support
   	else if (ctrl.selectionStart || ctrl.selectionStart == '0')
   		CaretPos = ctrl.selectionStart;
   	return (CaretPos);
}


RE.placeCaretAtEnd = function(el) {
    el.focus();
    if (typeof window.getSelection != "undefined"
            && typeof document.createRange != "undefined") {
        var range = document.createRange();
        range.selectNodeContents(el);
        range.collapse(false);
        var sel = window.getSelection();
        sel.removeAllRanges();
        sel.addRange(range);
    } else if (typeof document.body.createTextRange != "undefined") {
        var textRange = document.body.createTextRange();
        textRange.moveToElementText(el);
        textRange.collapse(false);
        textRange.select();
    }
}

RE.removeFormat = function() {
    execCommand('removeFormat', false, null);
}



// Event Listeners
//RE.editor.addEventListener("input", function() {
//    RE.callback();
//});

RE.deleteItem = function() {
    console.log('invoke deleteItem..........');
    RE.updateContent();

}

RE.notifyInsertImage = function(id) {
    console.log('invoke insertImage..........id = ' + id);
    RE.callback();
//    window.location.href="re-state://insertImage?id=" + id;
}

RE.updateContent = function() {
    console.log('invoke updateContent..........');
    window.location.href="re-state://updateContent?content=" + encodeURI(RE.getHtml());
}

RE.editor.addEventListener("input", RE.callback);
RE.editor.addEventListener("keyup", function(e) {

    //RE.enabledEditingItems(e);
   var KEY_LEFT = 37, KEY_RIGHT = 39;
   if (e.which == KEY_LEFT || e.which == KEY_RIGHT || e.which ==8 || e.which == 13) {
       RE.enabledEditingItems(e);
   }
});
RE.editor.addEventListener("click", RE.enabledEditingItems);
//RE.getNodeName = function() {
//    var node = window.getSelection().baseNode.parentNode.nodeName;
//    //alert(node)
//    window.location.href="action://touchend?nodeName=" + node;
//}

//RE.editor.addEventListener("click", RE.enabledEditingItems);
//RE.channelContainer.addEventListener("click", function() {
//    window.location.href="action://channel_select";
//});
//
//
//RE.editor.addEventListener("focus", function() {
//    window.location.href="action://focus?eType=2";
//});

//RE.editor.addEventListener("touchend", function() {
//    window.location.href="action://touchend";
//});

//RE.editor.addEventListener('paste', function(e){
//	e.preventDefault();
//	var clipboard = e.clipboardData;
//	var content = clipboard.getData('text/plain');
//    RE.setHtml(content);
//
//    RE.updateContent();
//	console.log('Ctrl + V, pasting' + content);
//});

